##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##

##  Eryk Wdowiak
##  original:  05 Sep 2017
##  updated:   27 Feb 2018
##  
##  R script to examine effect of state minimum wage on employment
##  data available:
##    *  employment status by state -- BLS
##    *  state minimum wage data -- Vaghul and Zipperer 
##    *  average annual pay -- BLS QCEW
##    *  consumer price inflation -- BLS
##  
##  NOTE:  we have data on the 50 US states from 1976 to 2016
##  but the analysis here focuses on the years 2001-2016 because
##  those are the years for which we have average annual pay.

##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##

##  input file
infile <- "MinWage-vs-Employment_dataset.csv"
dta <- read.csv( infile )

##  calculate the log odds ratio
dta$lno_emp <- log(( dta$emp_rate)/(100 - dta$emp_rate))

##  compare minimum wage with average annual pay
##  no reason to annualize the wage (i.e. "40 hours/week * 50 weeks")
##  all that would do is add "log(2000)"
dta$ln_state_minw <- log( dta$state_minw )
dta$ln_avg_annl_pay <- log( dta$avg_annl_pay )

##  take the log of the CPI index
dta$ln_cpi_index <- log( dta$cpi_index )

##  divide population by 1000
##  (resolves an "integer overflow" issue in the regression weights)
dta$civpopMil <- dta$civpop / 1000 

##  strip spaces from state names -- for simplicity and correspondence
##  with the names of the dummy variables (which have stripped spaces)
dta$state <- sub("\\s","",dta$state)

##  list of state names
state_names <- c("Alabama","Alaska","Arizona","Arkansas","California","Colorado","Connecticut","Delaware",
                 "Florida","Georgia","Hawaii","Idaho","Illinois","Indiana","Iowa","Kansas","Kentucky","Louisiana",
                 "Maine","Maryland","Massachusetts","Michigan","Minnesota","Mississippi","Missouri","Montana",
                 "Nebraska","Nevada","NewHampshire","NewJersey","NewMexico","NewYork","NorthCarolina","NorthDakota",
                 "Ohio","Oklahoma","Oregon","Pennsylvania","RhodeIsland","SouthCarolina","SouthDakota","Tennessee",
                 "Texas","Utah","Vermont","Virginia","Washington","WestVirginia","Wisconsin","Wyoming")

##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##

##  now start our look at the minimum wage with summary statistics
##  classify states by average minimum wage 2001-2016

##  create table of employment rates, minimum wage rates and average pay
MinWage <- data.frame( emp_rate     = rep(NA,50)  ,
                       state_minw   = rep(NA,50)  ,
                       avg_annl_pay = rep(NA,50)  )
row.names( MinWage ) <- state_names
for (i in state_names ) {
  for (j in names( MinWage)) {
    MinWage[i,j] <- weighted.mean(x = dta[which(dta$state==i & dta$year > 2000),j] ,
                                  w = dta[which(dta$state==i & dta$year > 2000),c("civpopMil")] )
  }
}

##  classify states by average minimum wage
##  but examine the largest states separately

##  classify
lo_minw <- row.names( MinWage )[which( MinWage$state_minw < 6.40 )]
md_minw <- row.names( MinWage )[which( MinWage$state_minw >= 6.40 & MinWage$state_minw < 6.85 )]
hi_minw <- row.names( MinWage )[which( MinWage$state_minw >= 6.85 )]

##  without function --  x without y
"%w/o%" <- function(x,y) x[!x %in% y] 

##  separate the largest states
large_states <- c("California","Texas","Florida","NewYork") 
lo_minw <- lo_minw %w/o% large_states
md_minw <- md_minw %w/o% large_states
hi_minw <- hi_minw %w/o% large_states


##  make summary table
MinWageGrp <- data.frame( emp_rate     = rep(NA,7)  ,
                          state_minw   = rep(NA,7)  ,
                          avg_annl_pay = rep(NA,7)  )
row.names( MinWageGrp ) <- c("hi_minw","md_minw","lo_minw","Texas","California","NewYork","Florida")

for ( j in names( MinWageGrp )) {
  MinWageGrp["hi_minw",j] <- weighted.mean(x = dta[which(dta$state %in% hi_minw & dta$year > 2000),j] ,
                                           w = dta[which(dta$state %in% hi_minw & dta$year > 2000),c("civpopMil")] )
}
for ( j in names( MinWageGrp )) {
  MinWageGrp["md_minw",j] <- weighted.mean(x = dta[which(dta$state %in% md_minw & dta$year > 2000),j] ,
                                           w = dta[which(dta$state %in% md_minw & dta$year > 2000),c("civpopMil")] )
}
for ( j in names( MinWageGrp )) {
  MinWageGrp["lo_minw",j] <- weighted.mean(x = dta[which(dta$state %in% lo_minw & dta$year > 2000),j] ,
                                           w = dta[which(dta$state %in% lo_minw & dta$year > 2000),c("civpopMil")] )
}
for (i in c("Texas","California","NewYork","Florida")) {
  for (j in names( MinWageGrp)) {
    MinWageGrp[i,j] <- weighted.mean(x = dta[which(dta$state==i & dta$year > 2000),j] ,
                                     w = dta[which(dta$state==i & dta$year > 2000),c("civpopMil")] )
  }
}

##  text for summary statistics
mk_hie <- function( states , grp ) {
  hie <- length( which( row.names(MinWage) %in% states & MinWage$emp_rate >= 60.00 ) )
  len <- length( which( row.names(MinWage) %in% states ))
  pce <- round( 100 * hie / len )
  ote <- paste(hie,"of the",len,grp,"minimum wage states (",sep=" ")
  ote <- paste(ote,pce," percent)\n",sep="")
  ote <- paste(ote,"had an employment rate at or above 60.0 percent\n\n",sep="") 
  ote
}

##  print it all out
print_MinWage <- function() {

  ote_himw <- mk_hie( states = c(hi_minw,"California","NewYork") , grp = "high" )
  ote_mdmw <- mk_hie( states = c(md_minw,"Florida")  , grp = "medium" )
  ote_lomw <- mk_hie( states = c(lo_minw,"Texas")    , grp = "low"    )
  
  cat("\n\n============================================================\n\n")
  cat("population weighted averages, 46 states, 2001-2016\n\n")
  print( round( MinWageGrp[c(1:3),] ,2))
  cat("\n")
  cat("averages for the 4 largest states, 2001-2016\n\n") 
  print( round( MinWageGrp[c("Texas","California","NewYork","Florida"),] ,2))
  cat("\n\n============================================================\n\n")
  cat("         $6.85 <= minimum wage\n\n")
  cat(ote_himw)
  print( round( MinWageGrp[c("California","NewYork"),] ,2))
  cat("\n")
  hi_minw_table <- MinWage[ which( row.names(MinWage) %in% hi_minw ),]
  print( round( hi_minw_table[order(hi_minw_table$emp_rate,decreasing=TRUE),],2))
  cat("\n\n============================================================\n\n")
  cat("           $6.40 <= minimum wage < $6.85\n\n")
  cat(ote_mdmw)
  print( round( MinWageGrp[c("Florida"),] ,2))
  cat("\n")
  md_minw_table <- MinWage[ which( row.names(MinWage) %in% md_minw ),]
  print( round( md_minw_table[order(md_minw_table$emp_rate,decreasing=TRUE),],2))
  cat("\n\n============================================================\n\n")
  cat("                 minimum wage < $6.40\n\n")
  cat(ote_lomw)
  print( round( MinWageGrp[c("Texas"),] ,2))
  cat("\n")
  lo_minw_table <- MinWage[ which( row.names(MinWage) %in% lo_minw ),]
  print( round( lo_minw_table[order(lo_minw_table$emp_rate,decreasing=TRUE),],2))
  cat("\n\n============================================================\n\n")
}
print_MinWage()

##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##

##  the Confederacy is well represented among the low minimum wage states
##  so we will also take another look at the American Civil War, 1861-1865
free_states <- c("Pennsylvania","NewJersey","Connecticut","Massachusetts","NewHampshire",
                 "NewYork","RhodeIsland","Vermont","Ohio","Indiana","Illinois","Maine","Michigan",
                 "Iowa","Wisconsin","California","Minnesota","Oregon","Kansas","Nevada")
border_states <- c("Delaware","Maryland","Kentucky","Missouri","WestVirginia")
confederacy <- c("SouthCarolina","Mississippi","Florida","Alabama","Georgia","Louisiana","Texas",
                 "Virginia","Arkansas","Tennessee","NorthCarolina")
new_states <- c("Nebraska","Colorado","NorthDakota","SouthDakota","Montana","Washington","Idaho",
                "Wyoming","Utah","Oklahoma","NewMexico","Arizona","Alaska","Hawaii")

print_confederacy_lominw <- function() {
  cat("\n============================================================\n\n")
  cat("the Confederacy is well represented among the low minimum wage states:\n\n")
  print( confederacy[ confederacy %in% c(lo_minw,"Texas") ] ) 
  cat("\n")
  cat("only two Confederate States are not among the low minimum wage states:\n\n")
  print( confederacy[ ! confederacy %in% c(lo_minw,"Texas") ] ) 
  cat("\n============================================================\n\n")
}
print_confederacy_lominw()

##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##

##  make plots of minimum wage, employment rate and average annual pay over time

##  let's make some lists of variables to plot
EmpTables <- list()
StateLists <- list( hi_minw = hi_minw , md_minw = md_minw , lo_minw = lo_minw ,
                    Texas = "Texas" , California  = "California" ,
                    NewYork  = "NewYork" , Florida = "Florida" ,
                    free_states = free_states , border_states = border_states , 
                    confederacy = confederacy , new_states = new_states )

for ( GrpName in names( StateLists )) {

  EmpTables[[GrpName]] <- data.frame( year       = c(2001:2016) , state_minw = rep(NA,16) , 
                                          emp_rate     = rep(NA,16) , lf_part    = rep(NA,16) ,
                                          unemp_rate   = rep(NA,16) , 
                                          avg_annl_pay = rep(NA,16) , cpi_index  = rep(NA,16) )
  row.names( EmpTables[[GrpName]] ) <- paste(c(2001:2016))

  for (vb in c("state_minw","emp_rate","lf_part","unemp_rate","avg_annl_pay","cpi_index")) {
    for (yr in c(2001:2016) ) {
      GrpOfStates <- dta$state %in% StateLists[[GrpName]]
      GrpAndYear <- which(GrpOfStates & dta$year == yr)
      EmpTables[[GrpName]][paste(yr),vb] <- weighted.mean(
                                                  x = dta[GrpAndYear,vb] ,
                                                  w = dta[GrpAndYear,c("civpopMil")])
    }
  }
}

##  ##  ##  ##  ##  ##  ##  ##  ##  ##

##  functions to make plotting easy-ish
easyplot <- function( xx , yy , pch , main , ylim , ylab , yseq ) {
  plot( xx , yy , type = "b" , axes = FALSE ,
        pch = pch , main = main , xlim = c(2000,2016) , xlab = "" ,
        ylim = ylim ,  ylab = ylab )
  axis( 1 , at = seq(2000,2016,2) ) 
  axis( 2 , at = yseq , pos = 2000 )   
}
easypoints <- function( xx , yy , pch ) {
  points( xx , yy , type = "b" , pch = pch )
}

##  ##  ##  ##  ##  ##  ##  ##  ##  ##

##  employment rate, 46 states
#svg(filename="images/st-min-wage_46-states_emp-rate.svg" , width = 9 , height = 6 ) ; par(cex=1.1)
png(filename="images/st-min-wage_46-states_emp-rate.png" , width=810, height=540) ; par(cex=1.35)
easyplot(  xx = EmpTables$hi_minw$year , yy = EmpTables$hi_minw$emp_rate , pch="H" ,
           main = "Employment Rate, 46 states" , ylab = "Employment Rate" ,
           ylim = c(56,66) , yseq = seq(56,66,2) )
easypoints( xx = EmpTables$md_minw$year , yy = EmpTables$md_minw$emp_rate , pch="M")
easypoints( xx = EmpTables$lo_minw$year , yy = EmpTables$lo_minw$emp_rate , pch="L")
legend("bottomleft", lwd=c(1,1,1), inset=0.075, pch=c("H","M","L"),
       legend=c("\"high\" min. wage states","\"medium\" min. wage states","\"low\" min. wage states"))
dev.off()

###  labor force participation rate, 46 states
##svg(filename="images/st-min-wage_46-states_lf-part.svg" , width = 9 , height = 6 ) ; par(cex=1.1)
#png(filename="images/st-min-wage_46-states_lf-part.png" , width=810, height=540) ; par(cex=1.35)
#easyplot(  xx = EmpTables$hi_minw$year , yy = EmpTables$hi_minw$lf_part, pch="H" ,
#           main = "Labor Force Participation Rate, 46 states" , ylab = "Labor Force Participation Rate" ,
#           ## ylim = c(57,69) , yseq = seq(57,69,3) )
#           ylim = c(60,70) , yseq = seq(60,70,2) )
#easypoints( xx = EmpTables$md_minw$year , yy = EmpTables$md_minw$lf_part, pch="M")
#easypoints( xx = EmpTables$lo_minw$year , yy = EmpTables$lo_minw$lf_part, pch="L")
#legend("bottomleft", lwd=c(1,1,1), inset=0.075, pch=c("H","M","L"),
#       legend=c("\"high\" min. wage states","\"medium\" min. wage states","\"low\" min. wage states"))
#dev.off()

###  unemployment rate, 46 states
##svg(filename="images/st-min-wage_46-states_unemp-rate.svg" , width = 9 , height = 6 ) ; par(cex=1.1)
#png(filename="images/st-min-wage_46-states_unemp-rate.png" , width=810, height=540) ; par(cex=1.35)
#easyplot(  xx = EmpTables$hi_minw$year , yy = EmpTables$hi_minw$unemp_rate, pch="H" ,
#           main = "Unemployment Rate, 46 states" , ylab = "Unemployment Rate" ,
#           ## ylim = c(4,10) , yseq = seq(4,10,2) )
#           ylim = c(3,11) , yseq = seq(3,11,2) )
#easypoints( xx = EmpTables$md_minw$year , yy = EmpTables$md_minw$unemp_rate, pch="M")
#easypoints( xx = EmpTables$lo_minw$year , yy = EmpTables$lo_minw$unemp_rate, pch="L")
#legend("topleft", lwd=c(1,1,1), inset=0.075, pch=c("H","M","L"),
#       legend=c("\"high\" min. wage states","\"medium\" min. wage states","\"low\" min. wage states"))
#dev.off()

###  average annual pay, 46 states
##svg(filename="images/st-min-wage_46-states_avg-annl-pay.svg" , width = 9 , height = 6 ) ; par(cex=1.1)
#png(filename="images/st-min-wage_46-states_avg-annl-pay.png" , width=810, height=540) ; par(cex=1.35)
#easyplot(  xx = EmpTables$hi_minw$year , yy = EmpTables$hi_minw$avg_annl_pay , pch="H" ,
#           main = "Average Annual Pay, 46 states" , ylab = "Average Annual Pay" ,
#           ## ylim = c(30000,70000) , yseq = seq(30000,70000,10000) )
#           ylim = c(30000,60000) , yseq = seq(30000,60000,10000) )
#easypoints( xx = EmpTables$md_minw$year , yy = EmpTables$md_minw$avg_annl_pay , pch="M")
#easypoints( xx = EmpTables$lo_minw$year , yy = EmpTables$lo_minw$avg_annl_pay , pch="L")
#legend("topleft", lwd=c(1,1,1), inset=0.075, pch=c("H","M","L"),
#       legend=c("\"high\" min. wage states","\"medium\" min. wage states","\"low\" min. wage states"))
#dev.off()

##  minimum wage, 46 states
#svg(filename="images/st-min-wage_46-states_min-wage.svg", width = 9 , height = 6 ) ; par(cex=1.1)
png(filename="images/st-min-wage_46-states_min-wage.png", width=810, height=540) ; par(cex=1.35)
easyplot( xx = EmpTables$hi_minw$year , yy = EmpTables$hi_minw$state_minw , pch="H" ,
          main = "Minimum Wage, 46 states" , ylab = "Minimum Wage" ,
          ylim = c(4,10) , yseq = seq(4,10,2) )
easypoints( xx = EmpTables$md_minw$year , yy = EmpTables$md_minw$state_minw , pch="M")
easypoints( xx = EmpTables$lo_minw$year , yy = EmpTables$lo_minw$state_minw , pch="L")
legend("topleft", inset=0.075, lwd=c(1,1,1), pch=c("H","M","L"), 
       legend=c("\"high\" min. wage states","\"medium\" min. wage states","\"low\" min. wage states"))
dev.off()

##  ##  real average annual pay, 46 states
##  #svg(filename="images/st-min-wage_46-states_real-avg-annl-pay.svg" , width = 9 , height = 6 ) ; par(cex=1.1)
##  png(filename="images/st-min-wage_46-states_real-avg-annl-pay.png" , width=810, height=540) ; par(cex=1.35)
##  easyplot(  xx = EmpTables$hi_minw$year , pch="H" ,
##             yy = EmpTables$hi_minw$avg_annl_pay  / ( EmpTables$hi_minw$cpi_index / EmpTables$hi_minw$cpi_index[16] ) , 
##             main = "Real Average Annual Pay, 46 states" , ylab = "Real Average Annual Pay (in 2016 dollars)" ,
##             ylim = c(30000,70000) , yseq = seq(30000,70000,10000) )
##  easypoints( xx = EmpTables$md_minw$year , pch="M" ,
##              yy = EmpTables$md_minw$avg_annl_pay / ( EmpTables$md_minw$cpi_index / EmpTables$md_minw$cpi_index[16] ))
##  easypoints( xx = EmpTables$lo_minw$year , pch="L" ,
##              yy = EmpTables$lo_minw$avg_annl_pay / ( EmpTables$lo_minw$cpi_index / EmpTables$lo_minw$cpi_index[16] ))
##  legend("bottomright", lwd=c(1,1,1), inset=0.075, pch=c("H","M","L"),
##         legend=c("\"high\" min. wage states","\"medium\" min. wage states","\"low\" min. wage states"))
##  dev.off()

##  ##  real minimum wage, 46 states
##  #svg(filename="images/st-min-wage_46-states_real-min-wage.svg", width = 9 , height = 6 ) ; par(cex=1.1)
##  png(filename="images/st-min-wage_46-states_real-min-wage.png", width=810, height=540) ; par(cex=1.35)
##  easyplot( xx = EmpTables$hi_minw$year , pch="H",
##            yy = EmpTables$hi_minw$state_minw / ( EmpTables$hi_minw$cpi_index / EmpTables$hi_minw$cpi_index[16] ) ,
##            main = "Real Minimum Wage, 46 states" , ylab = "Real Minimum Wage (in 2016 dollars)" , 
##            ylim = c(4,10) , yseq = seq(4,10,2) )
##  easypoints( xx = EmpTables$md_minw$year , pch="M", 
##              yy = EmpTables$md_minw$state_minw / ( EmpTables$md_minw$cpi_index / EmpTables$md_minw$cpi_index[16] ))
##  easypoints( xx = EmpTables$lo_minw$year , pch="L", 
##              yy = EmpTables$lo_minw$state_minw / ( EmpTables$lo_minw$cpi_index / EmpTables$lo_minw$cpi_index[16] ))
##  legend("bottomright", inset=0.075, lwd=c(1,1,1), pch=c("H","M","L"),
##         legend=c("\"high\" min. wage states","\"medium\" min. wage states","\"low\" min. wage states"))
##  dev.off()

##  ##  ##  ##  ##  ##  ##  ##  ##  ##

##  employment rate, big four
#svg(filename="images/st-min-wage_big-four_emp-rate.svg", width = 9 , height = 6 ) ; par(cex=1.1)
png(filename="images/st-min-wage_big-four_emp-rate.png", width=810, height=540) ; par(cex=1.35)
easyplot( xx = EmpTables$Texas$year , yy = EmpTables$Texas$emp_rate , pch="T" ,
          main = "Employment Rate, 4 largest states" , ylab = "Employment Rate" ,
          ylim = c(54,66) , yseq = seq(54,66,3) )
easypoints( xx = EmpTables$NewYork$year    , yy = EmpTables$NewYork$emp_rate    , pch="N")
easypoints( xx = EmpTables$Florida$year    , yy = EmpTables$Florida$emp_rate    , pch="F")
easypoints( xx = EmpTables$California$year , yy = EmpTables$California$emp_rate , pch="C")
legend("bottomleft", inset=0.075, lwd=c(1,1,1,1), pch=c("T","C","N","F"), 
       legend=c("Texas","California","New York","Florida"))
dev.off()

###  labor force participation rate, big four
##svg(filename="images/st-min-wage_big-four_lf-part.svg" , width = 9 , height = 6 ) ; par(cex=1.1)
#png(filename="images/st-min-wage_big-four_lf-part.png" , width=810, height=540) ; par(cex=1.35)
#easyplot(  xx = EmpTables$Texas$year , yy = EmpTables$Texas$lf_part, pch="T" ,
#           main = "Labor Force Participation Rate, 4 largest states" , ylab = "Labor Force Participation Rate" ,
#           ylim = c(57,69) , yseq = seq(57,69,3) )
#easypoints( xx = EmpTables$California$year , yy = EmpTables$California$lf_part, pch="C")
#easypoints( xx = EmpTables$NewYork$year , yy = EmpTables$NewYork$lf_part, pch="N")
#easypoints( xx = EmpTables$Florida$year , yy = EmpTables$Florida$lf_part, pch="F")
#legend("bottomleft", inset=0.075, lwd=c(1,1,1,1), pch=c("T","C","N","F"), 
#       legend=c("Texas","California","New York","Florida"))
#dev.off()

###  unemployment rate, big four
##svg(filename="images/st-min-wage_big-four_unemp-rate.svg" , width = 9 , height = 6 ) ; par(cex=1.1)
#png(filename="images/st-min-wage_big-four_unemp-rate.png" , width=810, height=540) ; par(cex=1.35)
#easyplot(  xx = EmpTables$Texas$year , yy = EmpTables$Texas$unemp_rate, pch="T" ,
#           main = "Unemployment Rate, 4 largest states" , ylab = "Unemployment Rate" ,
#           ylim = c(3,13) , yseq = seq(3,13,2) )
#easypoints( xx = EmpTables$California$year , yy = EmpTables$California$unemp_rate, pch="C")
#easypoints( xx = EmpTables$NewYork$year , yy = EmpTables$NewYork$unemp_rate, pch="N")
#easypoints( xx = EmpTables$Florida$year , yy = EmpTables$Florida$unemp_rate, pch="F")
#legend("topleft", inset=0.075, lwd=c(1,1,1,1), pch=c("C","F","N","T"), 
#       legend=c("California","Florida","New York","Texas"))
#dev.off()

###  average annual pay, big four
##svg(filename="images/st-min-wage_big-four_avg-annl-pay.svg", width = 9 , height = 6 ) ; par(cex=1.1)
#png(filename="images/st-min-wage_big-four_avg-annl-pay.png", width=810, height=540) ; par(cex=1.35)
#easyplot( xx = EmpTables$California$year , pch="C",
#          yy = EmpTables$California$avg_annl_pay ,
#          main = "Average Annual Pay, 4 largest states" , ylab = "Average Annual Pay" ,
#          ylim = c(30000,70000) , yseq = seq(30000,70000,10000) )
#easypoints( xx = EmpTables$NewYork$year,  yy = EmpTables$NewYork$avg_annl_pay , pch="N" )
#easypoints( xx = EmpTables$Florida$year , yy = EmpTables$Florida$avg_annl_pay , pch="F" )
#easypoints( xx = EmpTables$Texas$year , yy = EmpTables$Texas$avg_annl_pay , pch="T" )
#legend("topleft", inset=0.075, lwd=c(1,1,1,1), pch=c("N","C","T","F"), 
#       legend=c("New York","California","Texas","Florida"))
#dev.off()

##  minimum wage, big four
#svg(filename="images/st-min-wage_big-four_min-wage.svg", width = 9 , height = 6 ) ; par(cex=1.1)
png(filename="images/st-min-wage_big-four_min-wage.png", width=810, height=540) ; par(cex=1.35)
easyplot( xx = EmpTables$California$year , yy = EmpTables$California$state_minw , pch="C" ,
      main = "Minimum Wage, 4 largest states" , ylab = "Minimum Wage"  ,
      ylim = c(4,10) , yseq = seq(4,10,2) )
easypoints( xx = EmpTables$NewYork$year    , yy = EmpTables$NewYork$state_minw , pch="N")
easypoints( xx = EmpTables$Florida$year    , yy = EmpTables$Florida$state_minw , pch="F")
easypoints( xx = EmpTables$Texas$year      , yy = EmpTables$Texas$state_minw   , pch="T")
legend("topleft", inset=0.075, lwd=c(1,1,1,1), pch=c("C","N","F","T"), 
       legend=c("California","New York","Florida","Texas"))
dev.off()

##  ##  real average annual pay, big four
##  #svg(filename="images/st-min-wage_big-four_real-avg-annl-pay.svg" , width = 9 , height = 6 ) ; par(cex=1.1)
##  png(filename="images/st-min-wage_big-four_real-avg-annl-pay.png" , width=810, height=540) ; par(cex=1.35)
##  easyplot(  xx = EmpTables$California$year , pch="C" ,
##             yy = EmpTables$California$avg_annl_pay  / ( EmpTables$California$cpi_index / EmpTables$California$cpi_index[16] ) , 
##             main = "Real Average Annual Pay, 4 largest states" , ylab = "Real Average Annual Pay (in 2016 dollars)" ,
##             ylim = c(30000,70000) , yseq = seq(30000,70000,10000) )
##  easypoints( xx = EmpTables$NewYork$year , pch="N" ,
##              yy = EmpTables$NewYork$avg_annl_pay / ( EmpTables$NewYork$cpi_index / EmpTables$NewYork$cpi_index[16] ))
##  easypoints( xx = EmpTables$Florida$year , pch="F" ,
##              yy = EmpTables$Florida$avg_annl_pay / ( EmpTables$Florida$cpi_index / EmpTables$Florida$cpi_index[16] ))
##  easypoints( xx = EmpTables$Texas$year , pch="T" ,
##              yy = EmpTables$Texas$avg_annl_pay / ( EmpTables$Texas$cpi_index / EmpTables$Texas$cpi_index[16] ))
##  legend("bottomright", inset=0.075, lwd=c(1,1,1,1), pch=c("N","C","T","F"), 
##         legend=c("New York","California","Texas","Florida"))
##  dev.off()

##  ##  real minimum wage, big four
##  #svg(filename="images/st-min-wage_big-four_real-min-wage.svg", width = 9 , height = 6 ) ; par(cex=1.1)
##  png(filename="images/st-min-wage_big-four_real-min-wage.png", width=810, height=540) ; par(cex=1.35)
##  easyplot( xx =  EmpTables$California$year ,  pch="C" ,
##            yy = EmpTables$California$state_minw / ( EmpTables$California$cpi_index / EmpTables$California$cpi_index[16] ) ,
##            main = "Real Minimum Wage, 4 largest states" , ylab = "Real Minimum Wage (in 2016 dollars)" ,           
##            ylim = c(4,10) , yseq = seq(4,10,2) )
##  easypoints( xx = EmpTables$NewYork$year , pch="N" , 
##              yy = EmpTables$NewYork$state_minw / ( EmpTables$NewYork$cpi_index / EmpTables$NewYork$cpi_index[16] ) )
##  easypoints( xx = EmpTables$Florida$year , pch="F",
##              yy = EmpTables$Florida$state_minw / ( EmpTables$Florida$cpi_index / EmpTables$Florida$cpi_index[16] ) )
##  easypoints( xx = EmpTables$Texas$year , pch="T",
##              yy = EmpTables$Texas$state_minw / ( EmpTables$Texas$cpi_index / EmpTables$Texas$cpi_index[16] ) )
##  legend("bottomright", inset=0.075, lwd=c(1,1,1,1), pch=c("C","N","F","T"), 
##         legend=c("California","New York","Florida","Texas"))
##  dev.off()

##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##

##  the confederacy is well represented among the low minimum wage states
##  so we will also take another look at the American Civil War, 1861-1865

##  employment rate, by civil war status
#svg(filename="images/st-min-wage_civil-war_emp-rate.svg" , width = 9 , height = 6 ) ; par(cex=1.1)
png(filename="images/st-min-wage_civil-war_emp-rate.png" , width=810, height=540) ; par(cex=1.35)
easyplot(  xx = EmpTables$confederacy$year    , yy = EmpTables$confederacy$emp_rate   , pch="C",
           main = "Employment Rate, by Civil War status" , ylab = "Employment Rate" ,
           ## ylim = c(54,66) , yseq = seq(54,66,3) )
          ylim = c(56,66) , yseq = seq(56,66,2) )
easypoints( xx = EmpTables$new_states$year    , yy = EmpTables$new_states$emp_rate    , pch="N")
easypoints( xx = EmpTables$free_states$year   , yy = EmpTables$free_states$emp_rate   , pch="F")
easypoints( xx = EmpTables$border_states$year , yy = EmpTables$border_states$emp_rate , pch="B")
legend("bottomleft", inset=0.075, lwd=c(1,1,1,1), pch=c("N","F","B","C"), 
       legend=c("New States","Free States","Border States","Confederacy"))
dev.off()

###  labor force participation rate, by civil war status
##svg(filename="images/st-min-wage_civil-war_lf-part.svg" , width = 9 , height = 6 ) ; par(cex=1.1)
#png(filename="images/st-min-wage_civil-war_lf-part.png" , width=810, height=540) ; par(cex=1.35)
#easyplot(  xx = EmpTables$confederacy$year    , yy = EmpTables$confederacy$lf_part  , pch="C",
#           main = "Labor Force Participation Rate, by Civil War status" , ylab = "Labor Force Participation Rate" ,
#           ## ylim = c(57,69) , yseq = seq(57,69,3) )
#           ylim = c(60,70) , yseq = seq(60,70,2) )
#easypoints( xx = EmpTables$new_states$year    , yy = EmpTables$new_states$lf_part    , pch="N")
#easypoints( xx = EmpTables$free_states$year   , yy = EmpTables$free_states$lf_part   , pch="F")
#easypoints( xx = EmpTables$border_states$year , yy = EmpTables$border_states$lf_part , pch="B")
#legend("bottomleft", inset=0.075, lwd=c(1,1,1,1), pch=c("N","F","B","C"), 
#       legend=c("New States","Free States","Border States","Confederacy"))       
#dev.off()

###  unemployment rate, by civil war status
##svg(filename="images/st-min-wage_civil-war_unemp-rate.svg" , width = 9 , height = 6 ) ; par(cex=1.1)
#png(filename="images/st-min-wage_civil-war_unemp-rate.png" , width=810, height=540) ; par(cex=1.35)
#easyplot(  xx = EmpTables$confederacy$year    , yy = EmpTables$confederacy$unemp_rate  , pch="C",
#           main = "Unemployment Rate, by Civil War status" , ylab = "Unemployment Rate" ,
#           ylim = c(3,11) , yseq = seq(3,11,2) )
#easypoints( xx = EmpTables$new_states$year    , yy = EmpTables$new_states$unemp_rate    , pch="N")
#easypoints( xx = EmpTables$free_states$year   , yy = EmpTables$free_states$unemp_rate   , pch="F")
#easypoints( xx = EmpTables$border_states$year , yy = EmpTables$border_states$unemp_rate , pch="B")
#legend("topleft", inset=0.075, lwd=c(1,1,1,1), pch=c("F","C","B","N"), 
#       legend=c("Free States","Confederacy","Border States","New States"))       
#dev.off()

###  average annual pay, by civil war status
##svg(filename="images/st-min-wage_civil-war_avg-annl-pay.svg" , width = 9 , height = 6 ) ; par(cex=1.1)
#png(filename="images/st-min-wage_civil-war_avg-annl-pay.png" , width=810, height=540) ; par(cex=1.35)
#easyplot(  xx = EmpTables$confederacy$year   , yy = EmpTables$confederacy$avg_annl_pay   , pch="C",
#           main = "Average Annual Pay, by Civil War status" , ylab = "Average Annual Pay" ,
#           ## ylim = c(30000,70000) , yseq = seq(30000,70000,10000) )
#           ylim = c(30000,60000) , yseq = seq(30000,60000,10000) )
#easypoints( xx = EmpTables$new_states$year    , yy = EmpTables$new_states$avg_annl_pay    , pch="N")
#easypoints( xx = EmpTables$free_states$year   , yy = EmpTables$free_states$avg_annl_pay   , pch="F")
#easypoints( xx = EmpTables$border_states$year , yy = EmpTables$border_states$avg_annl_pay , pch="B")
#legend("topleft", inset=0.075, lwd=c(1,1,1,1), pch=c("F","B","C","N"), 
#       legend=c("Free States","Border States","Confederacy","New States"))
#dev.off()

##  minimum wage, by civil war status
#svg(filename="images/st-min-wage_civil-war_min-wage.svg", width = 9 , height = 6 ) ; par(cex=1.1)
png(filename="images/st-min-wage_civil-war_min-wage.png", width=810, height=540) ; par(cex=1.35)
easyplot( xx = EmpTables$confederacy$year     , yy = EmpTables$confederacy$state_minw   , pch="C",
      main = "Minimum Wage, by Civil War status" , ylab = "Minimum Wage"  ,
      ylim = c(4,10) , yseq = seq(4,10,2) )
easypoints( xx = EmpTables$new_states$year    , yy = EmpTables$new_states$state_minw    , pch="N")
easypoints( xx = EmpTables$free_states$year   , yy = EmpTables$free_states$state_minw   , pch="F")
easypoints( xx = EmpTables$border_states$year , yy = EmpTables$border_states$state_minw , pch="B")
legend("topleft", inset=0.075, lwd=c(1,1,1,1), pch=c("F","N","B","C"), 
       legend=c("Free States","New States","Border States","Confederacy"))
dev.off()

##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##

##  now let's run some regressions

##  state dummies -- for formula
stdums <- paste( "Alabama","Alaska","Arizona","Arkansas",
                 ##  "California",
                 "Colorado","Connecticut","Delaware",
                 "Florida","Georgia","Hawaii","Idaho","Illinois","Indiana","Iowa","Kansas","Kentucky","Louisiana",
                 "Maine","Maryland","Massachusetts","Michigan","Minnesota","Mississippi","Missouri","Montana",
                 "Nebraska","Nevada","NewHampshire","NewJersey","NewMexico","NewYork","NorthCarolina","NorthDakota",
                 "Ohio","Oklahoma","Oregon","Pennsylvania","RhodeIsland","SouthCarolina","SouthDakota","Tennessee",
                 "Texas","Utah","Vermont","Virginia","Washington","WestVirginia","Wisconsin","Wyoming", sep=" + ")

##  year dummies -- for formula
yrdums_early <- paste("yr1976","yr1977","yr1978","yr1979","yr1980",
                      "yr1981","yr1982","yr1983","yr1984","yr1985",
                      "yr1986","yr1987","yr1988","yr1989","yr1990",
                      "yr1991","yr1992","yr1993","yr1994","yr1995",
                      "yr1996","yr1997","yr1998","yr1999","yr2000", sep=" + ") 

yrdums_later <- paste("yr2001","yr2002","yr2003","yr2004","yr2005",
                      "yr2006","yr2007","yr2008","yr2009","yr2010",
                      "yr2011","yr2012","yr2013","yr2014", sep=" + ")
                      ## "yr2015", sep=" + ") ## ,"yr2016"

##  create a function to run weighted least squares
wls <- function( formula , data ) {
  
  ##  first-step regression
  StepOne <- lm( formula = formula , data = data )
  
  ##  compute the weights for step two
  data$StepOneProbs <- exp(StepOne$fitted.values ) / ( 1 + exp(StepOne$fitted.values))
  data$StepTwoWghts <- data$civpopMil * data$StepOneProbs * (1-data$StepOneProbs) 
  
  ##  now run the second-step
  Results <- lm( formula = formula , data = data , weights = StepTwoWghts )
  
  list( Results = Results , formula = formula , data = data )
}

##  function to run F-test -- omitting "ln_state_minw"
mk_Ftest <- function( rsults , formula ) {
  
  ##  omit "ln_state_minw" and conduct an F-test
  rdta <- rsults$model ;  rdta$weights <- rdta[["(weights)"]]
  omit_fmla <- sub("ln_state_minw \\+ ","",formula)
  omit_mdl  <- lm( formula = omit_fmla, data = rdta , weights = weights )
  
  ##  compute both R^2 and F-stat
  urR2 <- summary( rsults )$r.squared  ; rsR2 <- summary( omit_mdl )$r.squared 
  Fstat <- ((urR2 - rsR2)/1) / ((1-urR2)/( rsults$df.residual ))
  pval <- pf( q = Fstat , df1 = 1 , df2 = rsults$df.residual , lower.tail = FALSE )
  Ftest <- list( Fstat = Fstat , DFnum = 1 , DFden = rsults$df.residual , pval = pval )

  ##  return result
  Ftest
}
##  print the result
print_Ftest <- function( Ftest ) {
  ot_F <- paste("F-test of null hypothesis:  beta[\"ln_state_minw\"] = 0\n",sep="")
  ot_F <- paste(ot_F,"F stat:",round(Ftest$Fstat,2),"on",Ftest$DFnum,"and",Ftest$DFden,"DF\n",sep=" ")
  ot_F <- paste(ot_F,"p val.:",round(Ftest$pval,4),"\n",sep=" ")
  ot_F
}

##  regression models
rmodel_apay <- paste( "lno_emp ~ cpi_inflation + ln_state_minw + ln_avg_annl_pay + ln_cpi_index " , stdums , yrdums_later , sep = " + " )
rmodel_shrt <- paste( "lno_emp ~ cpi_inflation + ln_state_minw + ln_cpi_index " , stdums, yrdums_later , sep = " + " )
#rmodel_long <- paste( "lno_emp ~ cpi_inflation + ln_state_minw + ln_cpi_index " , stdums , yrdums_early , yrdums_later , sep = " + " )

##  let's also examine the relationship between minimum wage and average pay
pmodel_apayy <- paste( "ln_avg_annl_pay ~ ln_state_minw + ln_cpi_index " , yrdums_later , sep = " + " )
pmodel_apays <- paste( "ln_avg_annl_pay ~ ln_state_minw + ln_cpi_index " , stdums , yrdums_later , sep = " + " )

##  run the regressions
yrs_later <- which(dta$year > 2000)
tswls_apay <- wls( formula = rmodel_apay , data = dta[yrs_later,] )
tswls_shrt <- wls( formula = rmodel_shrt , data = dta[yrs_later,] )
#tswls_long <- wls( formula = rmodel_long , data = dta )
wls_apmwy <- lm( formula = pmodel_apayy, data = dta[yrs_later,] , weights = civpopMil )
wls_apmws <- lm( formula = pmodel_apays, data = dta[yrs_later,] , weights = civpopMil )

##  run the F-tests
F_apay  <- mk_Ftest( rsults = tswls_apay$Results , formula = tswls_apay$formula )
F_shrt  <- mk_Ftest( rsults = tswls_shrt$Results , formula = tswls_shrt$formula )
#F_long  <- mk_Ftest( rsults = tswls_long$Results , formula = tswls_long$formula )
F_apmwy <- mk_Ftest( rsults = wls_apmwy , formula = pmodel_apayy )
F_apmws <- mk_Ftest( rsults = wls_apmws , formula = pmodel_apays )

##  summarize the regressions
summarize_regressions <- function() {
  cat("\n============================================================\n\n")
  cat("2-step WLS with state and year fixed effects, 2001-2016\n\n") ;
  cat("  dependent:  log odds of employment\n")
  cat("explanatory:  CPI inflation, minimum wage, average annual pay\n") ; 
  print( summary( tswls_apay$Results ) )
  cat( print_Ftest( F_apay ) , "\n")
  cat("\n============================================================\n\n")
  cat("2-step WLS with state and year fixed effects, 2001-2016\n\n") ;
  cat("  dependent:  log odds of employment\n")
  cat("explanatory:  CPI inflation, minimum wage\n") ; 
  print( summary( tswls_shrt$Results ) )
  cat( print_Ftest( F_shrt ) , "\n")
  cat("\n============================================================\n\n")
  # cat("2-step WLS with state and year fixed effects, 1976-2016\n\n") ;
  # cat("  dependent:  log odds of employment\n")
  # cat("explanatory:  CPI inflation, minimum wage\n") ;   
  # print( summary( tswls_long$Results ) )
  # cat( print_Ftest( F_long ) , "\n")
  # cat("\n============================================================\n\n")
  cat("WLS with year fixed effects, 2001-2016\n\n") ;
  cat("  dependent:  log of average annual pay\n") ; 
  cat("explanatory:  consumer price level, minimum wage\n") ;
  print( summary( wls_apmwy) )
  cat( print_Ftest( F_apmwy ) , "\n")
  cat("\n============================================================\n\n")
  cat("WLS with state and year fixed effects, 2001-2016\n\n") ;
  cat("  dependent:  log of average annual pay\n") ; 
  cat("explanatory:  consumer price level, minimum wage\n") ;
  print( summary( wls_apmws) )
  cat( print_Ftest( F_apmws ) , "\n")
  cat("\n============================================================\n\n")
}
summarize_regressions()

##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##

##  that's all folks!
q("no")

##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  ##  

##  "state","year",
##  "civpop","labor_force","lf_part","employed","emp_rate","unemployed","unemp_rate",
##  "fedrl_minw","state_minw","avg_annl_pay","cpi_index","cpi_inflation",
##  "Alabama","Alaska","Arizona","Arkansas","California","Colorado","Connecticut","Delaware",
##  "Florida","Georgia","Hawaii","Idaho","Illinois","Indiana","Iowa","Kansas","Kentucky","Louisiana",
##  "Maine","Maryland","Massachusetts","Michigan","Minnesota","Mississippi","Missouri","Montana",
##  "Nebraska","Nevada","NewHampshire","NewJersey","NewMexico","NewYork","NorthCarolina","NorthDakota",
##  "Ohio","Oklahoma","Oregon","Pennsylvania","RhodeIsland","SouthCarolina","SouthDakota","Tennessee",
##  "Texas","Utah","Vermont","Virginia","Washington","WestVirginia","Wisconsin","Wyoming",
##  "yr1976","yr1977","yr1978","yr1979","yr1980","yr1981","yr1982","yr1983","yr1984","yr1985",
##  "yr1986","yr1987","yr1988","yr1989","yr1990","yr1991","yr1992","yr1993","yr1994","yr1995",
##  "yr1996","yr1997","yr1998","yr1999","yr2000","yr2001","yr2002","yr2003","yr2004","yr2005",
##  "yr2006","yr2007","yr2008","yr2009","yr2010","yr2011","yr2012","yr2013","yr2014","yr2015","yr2016"
